@@ -272,7 +272,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
   // thread-safe init:
   dt_exif_init();
   char datadir[1024];
-  dt_get_user_config_dir (datadir,1024);
+  dt_util_get_user_config_dir (datadir,1024);
   char filename[1024];
   snprintf(filename, 1024, "%s/darktablerc", datadir);
 
@@ -285,6 +285,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
 #endif
   // has to go first for settings needed by all the others.
   darktable.conf = (dt_conf_t *)malloc(sizeof(dt_conf_t));
+  memset(darktable.conf, 0, sizeof(dt_conf_t));
   dt_conf_init(darktable.conf, filename);
 
   // get max lighttable thumbnail size:
@@ -301,7 +302,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
   gchar *conf_db = dt_conf_get_string("database");
   if (conf_db && conf_db[0] != '/')
   {
-    char *homedir = getenv ("HOME");
+    char *homedir = dt_util_get_home_dir(NULL);
     snprintf (dbfilename,2048,"%s/%s", homedir, conf_db);
     if (g_file_test (dbfilename, G_FILE_TEST_EXISTS))
     {
@@ -324,7 +325,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
   gchar *conf_cache = dt_conf_get_string("cachefile");
   if (conf_cache && conf_cache[0] != '/')
   {
-    char *homedir = getenv ("HOME");
+    char *homedir = dt_util_get_home_dir(NULL);
     snprintf (cachefilename,2048,"%s/%s",homedir, conf_cache);
     if (g_file_test (cachefilename,G_FILE_TEST_EXISTS))
     {
@@ -364,7 +365,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
     else       fprintf(stderr, "\n");
 #ifndef HAVE_GCONF
     fprintf(stderr, "[init] maybe your %s/darktablerc is corrupt?\n",datadir);
-    dt_get_datadir(dbfilename, 512);
+    dt_util_get_datadir(dbfilename, 512);
     fprintf(stderr, "[init] try `cp %s/darktablerc %s/darktablerc'\n", dbfilename,datadir);
 #else
     fprintf(stderr, "[init] check your /apps/darktable/database gconf entry!\n");
@@ -379,6 +380,7 @@ int dt_init(int argc, char *argv[], const int init_gui)
   dt_pthread_mutex_init(&(darktable.plugin_threadsafe), NULL);
 
   darktable.control = (dt_control_t *)malloc(sizeof(dt_control_t));
+  memset(darktable.control, 0, sizeof(dt_control_t));
   if(init_gui)
   {
     dt_control_init(darktable.control);
@@ -405,21 +407,26 @@ int dt_init(int argc, char *argv[], const int init_gui)
   darktable.collection = dt_collection_new(NULL);
 
   darktable.opencl = (dt_opencl_t *)malloc(sizeof(dt_opencl_t));
+  memset(darktable.opencl, 0, sizeof(dt_opencl_t));
   dt_opencl_init(darktable.opencl, argc, argv);
 
   darktable.blendop = (dt_blendop_t *)malloc(sizeof(dt_blendop_t));
+  memset(darktable.blendop, 0, sizeof(dt_blendop_t));
   dt_develop_blend_init(darktable.blendop);
 
   darktable.points = (dt_points_t *)malloc(sizeof(dt_points_t));
+  memset(darktable.points, 0, sizeof(dt_points_t));
   dt_points_init(darktable.points, dt_get_num_threads());
 
   int thumbnails = dt_conf_get_int ("mipmap_cache_thumbnails");
   thumbnails = MIN(1000000, MAX(20, thumbnails));
 
   darktable.mipmap_cache = (dt_mipmap_cache_t *)malloc(sizeof(dt_mipmap_cache_t));
+  memset(darktable.mipmap_cache, 0, sizeof(dt_mipmap_cache_t));
   dt_mipmap_cache_init(darktable.mipmap_cache, thumbnails);
 
   darktable.image_cache = (dt_image_cache_t *)malloc(sizeof(dt_image_cache_t));
+  memset(darktable.image_cache, 0, sizeof(dt_image_cache_t));
   dt_image_cache_init(darktable.image_cache, MIN(10000, MAX(500, thumbnails)), load_cached);
 
   // The GUI must be initialized before the views, because the init()
@@ -429,10 +436,13 @@ int dt_init(int argc, char *argv[], const int init_gui)
   if(init_gui)
   {
     darktable.gui = (dt_gui_gtk_t *)malloc(sizeof(dt_gui_gtk_t));
+    memset(darktable.gui,0,sizeof(dt_gui_gtk_t));
     if(dt_gui_gtk_init(darktable.gui, argc, argv)) return 1;
   }
+  else darktable.gui = NULL;
 
   darktable.view_manager = (dt_view_manager_t *)malloc(sizeof(dt_view_manager_t));
+  memset(darktable.view_manager, 0, sizeof(dt_view_manager_t));
   dt_view_manager_init(darktable.view_manager);
 
   // load the darkroom mode plugins once:
@@ -441,12 +451,14 @@ int dt_init(int argc, char *argv[], const int init_gui)
   if(init_gui)
   {
     darktable.lib = (dt_lib_t *)malloc(sizeof(dt_lib_t));
+    memset(darktable.lib, 0, sizeof(dt_lib_t));
     dt_lib_init(darktable.lib);
 
     dt_control_load_config(darktable.control);
     g_strlcpy(darktable.control->global_settings.dbname, filename, 512); // overwrite if relocated.
 
     darktable.imageio = (dt_imageio_t *)malloc(sizeof(dt_imageio_t));
+    memset(darktable.imageio, 0, sizeof(dt_imageio_t));
     dt_imageio_init(darktable.imageio);
   }
 
@@ -560,24 +572,34 @@ int dt_init(int argc, char *argv[], const int init_gui)
 void dt_cleanup()
 {
   dt_ctl_switch_mode_to(DT_MODE_NONE);
+  const int init_gui = (darktable.gui != NULL);
 
-  dt_control_write_config(darktable.control);
-  dt_control_shutdown(darktable.control);
+  if(init_gui)
+  {
+    dt_control_write_config(darktable.control);
+    dt_control_shutdown(darktable.control);
 
-  dt_lib_cleanup(darktable.lib);
-  free(darktable.lib);
+    dt_lib_cleanup(darktable.lib);
+    free(darktable.lib);
+  }
   dt_view_manager_cleanup(darktable.view_manager);
   free(darktable.view_manager);
-  dt_imageio_cleanup(darktable.imageio);
-  free(darktable.imageio);
-  dt_gui_gtk_cleanup(darktable.gui);
-  free(darktable.gui);
+  if(init_gui)
+  {
+    dt_imageio_cleanup(darktable.imageio);
+    free(darktable.imageio);
+    dt_gui_gtk_cleanup(darktable.gui);
+    free(darktable.gui);
+  }
   dt_image_cache_cleanup(darktable.image_cache);
   free(darktable.image_cache);
   dt_mipmap_cache_cleanup(darktable.mipmap_cache);
   free(darktable.mipmap_cache);
-  dt_control_cleanup(darktable.control);
-  free(darktable.control);
+  if(init_gui)
+  {
+    dt_control_cleanup(darktable.control);
+    free(darktable.control);
+  }
   dt_conf_cleanup(darktable.conf);
   free(darktable.conf);
   dt_points_cleanup(darktable.points);
@@ -636,108 +658,6 @@ void *dt_alloc_align(size_t alignment, size_t size)
 #endif
 }
 
-void
-dt_get_user_config_dir (char *data, size_t bufsize)
-{
-  g_snprintf (data,bufsize,"%s/.config/darktable",getenv("HOME"));
-  if (g_file_test (data,G_FILE_TEST_EXISTS)==FALSE)
-    g_mkdir_with_parents (data,0700);
-}
-
-void
-dt_get_user_cache_dir (char *data, size_t bufsize)
-{
-  g_snprintf (data,bufsize,"%s/.cache/darktable",getenv("HOME"));
-  if (g_file_test (data,G_FILE_TEST_EXISTS)==FALSE)
-    g_mkdir_with_parents (data,0700);
-}
-
-
-void
-dt_get_user_local_dir (char *data, size_t bufsize)
-{
-  g_snprintf(data,bufsize,"%s/.local",getenv("HOME"));
-  if (g_file_test (data,G_FILE_TEST_EXISTS)==FALSE)
-    g_mkdir_with_parents (data,0700);
-
-}
-
-void dt_get_plugindir(char *datadir, size_t bufsize)
-{
-#if defined(__MACH__) || defined(__APPLE__)
-  gchar *curr = g_get_current_dir();
-  int contains = 0;
-  for(int k=0; darktable.progname[k] != 0; k++) if(darktable.progname[k] == '/')
-    {
-      contains = 1;
-      break;
-    }
-  if(darktable.progname[0] == '/') // absolute path
-    snprintf(datadir, bufsize, "%s", darktable.progname);
-  else if(contains) // relative path
-    snprintf(datadir, bufsize, "%s/%s", curr, darktable.progname);
-  else
-  {
-    // no idea where we have been called. use compiled in path
-    g_free(curr);
-    snprintf(datadir, bufsize, "%s/darktable", DARKTABLE_LIBDIR);
-    return;
-  }
-  size_t len = MIN(strlen(datadir), bufsize);
-  char *t = datadir + len; // strip off bin/darktable
-  for(; t>datadir && *t!='/'; t--);
-  t--;
-  if(*t == '.' && *(t-1) != '.')
-  {
-    for(; t>datadir && *t!='/'; t--);
-    t--;
-  }
-  for(; t>datadir && *t!='/'; t--);
-  g_strlcpy(t, "/lib/darktable", bufsize-(t-datadir));
-  g_free(curr);
-#else
-  snprintf(datadir, bufsize, "%s/darktable", DARKTABLE_LIBDIR);
-#endif
-}
-
-void dt_get_datadir(char *datadir, size_t bufsize)
-{
-#if defined(__MACH__) || defined(__APPLE__)
-  gchar *curr = g_get_current_dir();
-  int contains = 0;
-  for(int k=0; darktable.progname[k] != 0; k++) if(darktable.progname[k] == '/')
-    {
-      contains = 1;
-      break;
-    }
-  if(darktable.progname[0] == '/') // absolute path
-    snprintf(datadir, bufsize, "%s", darktable.progname);
-  else if(contains) // relative path
-    snprintf(datadir, bufsize, "%s/%s", curr, darktable.progname);
-  else
-  {
-    // no idea where we have been called. use compiled in path
-    g_free(curr);
-    snprintf(datadir, bufsize, "%s", DARKTABLE_DATADIR);
-    return;
-  }
-  size_t len = MIN(strlen(datadir), bufsize);
-  char *t = datadir + len; // strip off bin/darktable
-  for(; t>datadir && *t!='/'; t--);
-  t--;
-  if(*t == '.' && *(t-1) != '.')
-  {
-    for(; t>datadir && *t!='/'; t--);
-    t--;
-  }
-  for(; t>datadir && *t!='/'; t--);
-  g_strlcpy(t, "/share/darktable", bufsize-(t-datadir));
-  g_free(curr);
-#else
-  snprintf(datadir, bufsize, "%s", DARKTABLE_DATADIR);
-#endif
-}
-
 void dt_show_times(const dt_times_t *start, const char *prefix, const char *suffix, ...)
 {
   dt_times_t end;
