@@ -25,6 +25,7 @@
 #include "config.h"
 #endif
 #include "common/dtpthread.h"
+#include "common/utility.h"
 #include <time.h>
 #include <sys/resource.h>
 #include <sys/time.h>
@@ -161,14 +162,6 @@ void dt_print(dt_debug_thread_t thread, const char *msg, ...);
 void dt_gettime_t(char *datetime, time_t t);
 void dt_gettime(char *datetime);
 void *dt_alloc_align(size_t alignment, size_t size);
-void dt_get_datadir(char *datadir, size_t bufsize);
-void dt_get_plugindir(char *datadir, size_t bufsize);
-/** get the user directory of darktable, ~/.config/darktable */
-void dt_get_user_config_dir(char *data, size_t bufsize);
-/** get the user directory of darktable, ~/.cache/darktable */
-void dt_get_user_cache_dir(char *data, size_t bufsize);
-/** get the user local directory , ~/.local */
-void dt_get_user_local_dir(char *data, size_t bufsize);
 
 static inline double dt_get_wtime(void)
 {
