@@ -90,7 +90,7 @@ void dt_image_cache_write(dt_image_cache_t *cache)
 
   char cachedir[1024];
   char dbfilename[1024];
-  dt_get_user_cache_dir(cachedir,1024);
+  dt_util_get_user_cache_dir(cachedir,1024);
   gchar *filename = dt_conf_get_string("cachefile");
 
   if(!filename || filename[0] == '\0') snprintf(dbfilename, 512, "%s/%s", cachedir,DT_IMAGE_CACHE_FILE_NAME);
@@ -216,7 +216,7 @@ int dt_image_cache_read(dt_image_cache_t *cache)
   dt_pthread_mutex_lock(&(cache->mutex));
   char cachedir[1024];
   char dbfilename[1024];
-  dt_get_user_cache_dir (cachedir,1024);
+  dt_util_get_user_cache_dir (cachedir,1024);
   gchar *filename = dt_conf_get_string ("cachefile");
   if(!filename || filename[0] == '\0') snprintf (dbfilename, 512, "%s/%s", cachedir, DT_IMAGE_CACHE_FILE_NAME);
   else if(filename[0] != '/')          snprintf (dbfilename, 512, "%s/%s", cachedir, filename);
@@ -395,7 +395,7 @@ static void _image_cache_backup()
 {
   char cachedir[1024];
   char dbfilename[1024];
-  dt_get_user_cache_dir (cachedir,1024);
+  dt_util_get_user_cache_dir (cachedir,1024);
   gchar *filename = dt_conf_get_string ("cachefile");
 
   if(!filename || filename[0] == '\0') snprintf (dbfilename, 1024, "%s/%s", cachedir, DT_IMAGE_CACHE_FILE_NAME);
@@ -413,7 +413,7 @@ static void _image_cache_restore()
 {
   char cachedir[1024];
   char dbfilename[1024];
-  dt_get_user_cache_dir(cachedir,1024);
+  dt_util_get_user_cache_dir(cachedir,1024);
   gchar *filename = dt_conf_get_string ("cachefile");
 
   if(!filename || filename[0] == '\0') snprintf (dbfilename, 512, "%s/%s", cachedir,DT_IMAGE_CACHE_FILE_NAME);
