@@ -1027,13 +1027,6 @@ has_ldr_extension(const char *filename)
   return ret;
 }
 
-/**
- * \brief Loads the image, attempting a variety of methods
- *
- * \param[out] img a dt_image_t to associate the loaded image with
- * \param[in] filename the filename containing the image contents
- * \return A dt_imageio_retval_t specifying the state of the image
- */
 dt_imageio_retval_t dt_imageio_open(dt_image_t *img, const char *filename)
 {
   dt_imageio_retval_t ret = DT_IMAGEIO_FILE_CORRUPTED;
@@ -1053,16 +1046,10 @@ dt_imageio_retval_t dt_imageio_open(dt_image_t *img, const char *filename)
       ret = dt_imageio_open_ldr(img, filename);
   if(ret == DT_IMAGEIO_OK) dt_image_cache_flush_no_sidecars(img);
   img->flags &= ~DT_IMAGE_THUMBNAIL;
+  img->dirty = 1;
   return ret;
 }
 
-/**
- * \brief Loads the image preview, attempting a variety of methods
- *
- * \param[out] img a dt_image_t to associate the loaded preview with
- * \param[in] filename the filename containing the image contents
- * \return A dt_imageio_retval_t specifying the state of the image
- */
 dt_imageio_retval_t dt_imageio_open_preview(dt_image_t *img, const char *filename)
 {
   dt_imageio_retval_t ret = DT_IMAGEIO_FILE_CORRUPTED;
@@ -1081,6 +1068,7 @@ dt_imageio_retval_t dt_imageio_open_preview(dt_image_t *img, const char *filenam
   if(ret != DT_IMAGEIO_OK && ret != DT_IMAGEIO_CACHE_FULL)      // Failsafing, if ldr magic test fails..
     ret = dt_imageio_open_ldr_preview(img, filename);
   if(ret == DT_IMAGEIO_OK) dt_image_cache_flush_no_sidecars(img);
+  img->dirty = 1;
   return ret;
 }
 
