@@ -24,8 +24,11 @@
 
 #define DT_OPENCL_MAX_PROGRAMS 256
 #define DT_OPENCL_MAX_KERNELS 512
-#define DT_OPENCL_EVENTLISTSIZE 128
+#define DT_OPENCL_EVENTLISTSIZE 256
 #define DT_OPENCL_EVENTNAMELENGTH 64
+#define DT_OPENCL_MAX_EVENTS 256
+
+#define DT_OPENCL_MEMORY_HEADROOM (256*1024*1024)
 
 #ifdef HAVE_OPENCL
 
@@ -37,6 +40,19 @@
 #include "common/dlopencl.h"
 #include "control/conf.h"
 
+
+/**
+ * Accounting information used for OpenCL events.
+ */
+typedef struct dt_opencl_eventtag_t
+{
+  cl_int retval;
+  cl_ulong timelapsed;
+  char tag[DT_OPENCL_EVENTNAMELENGTH];
+}
+dt_opencl_eventtag_t;
+
+
 /**
  * to support multi-gpu and mixed systems with cpu support,
  * we encapsulate devices and use separate command queues.
@@ -57,9 +73,12 @@ typedef struct dt_opencl_device_t
   int program_used[DT_OPENCL_MAX_PROGRAMS];
   int kernel_used [DT_OPENCL_MAX_KERNELS];
   cl_event *eventlist;
-  char *eventnames;
+  dt_opencl_eventtag_t *eventtags;
   int numevents;
+  int eventsconsolidated;
   int maxevents;
+  int lostevents;
+  cl_int summary;
 }
 dt_opencl_device_t;
 
@@ -112,12 +131,18 @@ void dt_opencl_free_kernel(const int kernel);
 /** return max size in sizes[3]. */
 int dt_opencl_get_max_work_item_sizes(const int dev, size_t *sizes);
 
+/** return max size per dimension in sizes[3] and max total size in workgroupsize */
+int dt_opencl_get_work_group_limits(const int dev, size_t *sizes, size_t *workgroupsize, unsigned long *localmemsize);
+
 /** attach arg. */
 int dt_opencl_set_kernel_arg(const int dev, const int kernel, const int num, const size_t size, const void *arg);
 
 /** launch kernel! */
 int dt_opencl_enqueue_kernel_2d(const int dev, const int kernel, const size_t *sizes);
 
+/** launch kernel with defined local size! */
+int dt_opencl_enqueue_kernel_2d_with_local(const int dev, const int kernel, const size_t *sizes, const size_t *local);
+
 /** check if opencl is inited */
 int dt_opencl_is_inited(void);
 
@@ -135,22 +160,38 @@ int dt_opencl_copy_device_to_host(const int devid, void *host, void *device, con
 
 int dt_opencl_read_host_from_device(const int devid, void *host, void *device, const int width, const int height, const int bpp);
 
+int dt_opencl_read_host_from_device_rowpitch(const int devid, void *host, void *device, const int width, const int height, const int rowpitch);
+
+int dt_opencl_read_host_from_device_raw(const int devid, void *host, void *device, const size_t *origin, const size_t *region, const int rowpitch, const int blocking);
+
 int dt_opencl_write_host_to_device(const int devid, void *host, void *device, const int width, const int height, const int bpp);
 
+int dt_opencl_write_host_to_device_rowpitch(const int devid, void *host, void *device, const int width, const int height, const int rowpitch);
+
+int dt_opencl_write_host_to_device_raw(const int devid, void *host, void *device, const size_t *origin, const size_t *region, const int rowpitch, const int blocking);
+
 void* dt_opencl_copy_host_to_device(const int devid, void *host, const int width, const int height, const int bpp);
 
+void* dt_opencl_copy_host_to_device_rowpitch(const int devid, void *host, const int width, const int height, const int bpp, const int rowpitch);
+
 void* dt_opencl_copy_host_to_device_constant(const int devid, const int size, void *host);
 
 int dt_opencl_enqueue_copy_image(const int devid, cl_mem src, cl_mem dst, size_t *orig_src, size_t *orig_dst, size_t *region);
 
 void* dt_opencl_alloc_device(const int devid, const int width, const int height, const int bpp);
 
-void* dt_opencl_alloc_device_use_host_pointer(const int devid, const int width, const int height, const int bpp, void *host);
+void* dt_opencl_alloc_device_use_host_pointer(const int devid, const int width, const int height, const int bpp, const int rowpitch, void *host);
+
+int dt_opencl_enqueue_copy_image_to_buffer(const int devid, cl_mem src_image, cl_mem dst_buffer, size_t *origin, size_t *region, size_t offset);
+
+int dt_opencl_enqueue_copy_buffer_to_image(const int devid, cl_mem src_buffer, cl_mem dst_image, size_t offset, size_t *origin, size_t *region);
+
+void* dt_opencl_alloc_device_buffer(const int devid, const int size);
 
 void dt_opencl_release_mem_object(void *mem);
 
 /** check if image size fit into limits given by OpenCL runtime */
-int dt_opencl_image_fits_device(const int devid, const size_t width, const size_t height);
+int dt_opencl_image_fits_device(const int devid, const size_t width, const size_t height, const unsigned bpp, const float factor, const size_t overhead);
 
 /** get global memory of device */
 cl_ulong dt_opencl_get_max_global_mem(const int devid);
@@ -167,10 +208,10 @@ void dt_opencl_events_wait_for(const int devid);
 
 /** Wait for events in eventlist to terminate, check for return status of events and
     report summary success info (CL_COMPLETE or last error code) */
-cl_int dt_opencl_events_flush(const int devid, const int retain);
+cl_int dt_opencl_events_flush(const int devid, const int reset);
 
 /** display OpenCL profiling information. If summary is not 0, try to generate summarized info for kernels */
-void dt_opencl_events_profiling(const int devid, int summary);
+void dt_opencl_events_profiling(const int devid, const int aggregated);
 
 #else
 #include <stdlib.h>
@@ -215,7 +256,11 @@ static inline int dt_opencl_create_kernel(const int program, const char *name)
 static inline void dt_opencl_free_kernel(const int kernel) {}
 static inline int  dt_opencl_get_max_work_item_sizes(const int dev, size_t *sizes)
 {
-  return 0;
+  return -1;
+}
+static inline int dt_opencl_get_work_group_limits(const int dev, size_t *sizes, size_t *workgroupsize, unsigned long *localmemsize)
+{
+  return -1;
 }
 static inline int dt_opencl_set_kernel_arg(const int dev, const int kernel, const size_t size, const void *arg)
 {
@@ -225,6 +270,10 @@ static inline int dt_opencl_enqueue_kernel_2d(const int dev, const int kernel, c
 {
   return -1;
 }
+static inline int dt_opencl_enqueue_kernel_2d_with_local(const int dev, const int kernel, const size_t *sizes, const size_t *local)
+{
+  return -1;
+}
 static inline int dt_opencl_is_inited(void)
 {
   return 0;
@@ -238,6 +287,10 @@ static inline int dt_opencl_update_enabled(void)
 {
   return 0;
 }
+static inline int dt_opencl_image_fits_device(const int devid, const size_t width, const size_t height, const unsigned bpp, const float factor, const size_t overhead)
+{
+  return 0;
+}
 static inline int dt_opencl_get_max_global_mem(const int devid)
 {
   return 0;
@@ -248,12 +301,12 @@ static inline void *dt_opencl_events_get_slot(const int devid, const char *tag)
   return NULL;
 }
 static inline void dt_opencl_events_reset(const int devid) {}
-static void dt_opencl_events_wait_for(const int devid) {}
-static inline int dt_opencl_events_flush(const int devid, const int retain)
+static inline void dt_opencl_events_wait_for(const int devid) {}
+static inline int dt_opencl_events_flush(const int devid, const int reset)
 {
-  return -1;
+  return 0;
 }
-void dt_opencl_events_profiling(const int devid, int summary) {}
+static inline void dt_opencl_events_profiling(const int devid, const int aggregated) {}
 #endif
 
 
