@@ -46,7 +46,7 @@
 #include <math.h>
 #include <string.h>
 #include <glib/gstdio.h>
-
+#include <gdk/gdkkeysyms.h>
 
 void dt_ctl_settings_default(dt_control_t *c)
 {
@@ -55,8 +55,8 @@ void dt_ctl_settings_default(dt_control_t *c)
   dt_conf_set_int  ("config_version", DT_CONFIG_VERSION);
   dt_conf_set_bool ("write_sidecar_files", TRUE);
   dt_conf_set_bool ("ask_before_delete", TRUE);
-  dt_conf_set_float("preview_subsample", .125f);
-  dt_conf_set_int  ("mipmap_cache_thumbnails", 30000);
+  dt_conf_set_float("preview_subsample", 1.f);
+  dt_conf_set_int  ("mipmap_cache_thumbnails", 1000);
   dt_conf_set_int  ("parallel_export", 1);
   dt_conf_set_int  ("cache_memory", 536870912);
   dt_conf_set_int  ("database_cache_quality", 89);
@@ -82,6 +82,7 @@ void dt_ctl_settings_default(dt_control_t *c)
 
   dt_conf_set_int  ("ui_last/combo_sort",     DT_LIB_SORT_FILENAME);
   dt_conf_set_int  ("ui_last/combo_filter",   DT_LIB_FILTER_STAR_1);
+  dt_conf_set_int  ("ui_last/initial_rating", DT_LIB_FILTER_STAR_1);
 
   // import settings
   dt_conf_set_string ("capture/camera/storage/basedirectory", "$(PICTURES_FOLDER)/Darktable");
@@ -96,6 +97,10 @@ void dt_ctl_settings_default(dt_control_t *c)
   dt_conf_set_int  ("plugins/collection/rating",            1);
   dt_conf_set_int  ("plugins/lighttable/collect/num_rules", 0);
 
+  // reasonable thumbnail res:
+  dt_conf_set_int  ("plugins/lighttable/thumbnail_size", 800);
+
+  // should be unused:
   dt_conf_set_float("gamma_linear", .1f);
   dt_conf_set_float("gamma_gamma", .45f);
 }
@@ -144,6 +149,14 @@ static void dt_control_sanitize_database()
   }
   sqlite3_finalize(stmt);
   sqlite3_finalize(innerstmt);
+
+  // temporary stuff for some ops, need this for some reason with newer sqlite3:
+  DT_DEBUG_SQLITE3_EXEC(darktable.db, "create temp table color_labels_temp (imgid integer primary key)", NULL, NULL, NULL);
+  DT_DEBUG_SQLITE3_EXEC(darktable.db, "create temp table tmp_selection (imgid integer)", NULL, NULL, NULL);
+  DT_DEBUG_SQLITE3_EXEC(darktable.db, "create temp table tagquery1 (tagid integer, name varchar, count integer)", NULL, NULL, NULL);
+  DT_DEBUG_SQLITE3_EXEC(darktable.db, "create temp table tagquery2 (tagid integer, name varchar, count integer)", NULL, NULL, NULL);
+  DT_DEBUG_SQLITE3_EXEC(darktable.db, "create temporary table temp_history as select * from history", NULL, NULL, NULL);
+  DT_DEBUG_SQLITE3_EXEC(darktable.db, "delete from temp_history", NULL, NULL, NULL);
 }
 
 int dt_control_load_config(dt_control_t *c)
@@ -457,12 +470,64 @@ create_tables:
 
 void dt_control_key_accelerators_on(struct dt_control_t *s)
 {
-  s->key_accelerators_on = 1;
+  guint state = darktable.control->key_accelerators_saved;
+  if(!s->key_accelerators_on)
+  {
+    s->key_accelerators_on = 1;
+
+    if(state & ACCELS_GLOBAL)
+      gtk_window_add_accel_group(
+          GTK_WINDOW(darktable.gui->widgets.main_window),
+          darktable.control->accels_global);
+    if(state & ACCELS_LIGHTTABLE)
+      gtk_window_add_accel_group(
+          GTK_WINDOW(darktable.gui->widgets.main_window),
+          darktable.control->accels_lighttable);
+    if(state & ACCELS_DARKROOM)
+      gtk_window_add_accel_group(
+          GTK_WINDOW(darktable.gui->widgets.main_window),
+          darktable.control->accels_darkroom);
+    if(state & ACCELS_CAPTURE)
+      gtk_window_add_accel_group(
+          GTK_WINDOW(darktable.gui->widgets.main_window),
+          darktable.control->accels_capture);
+    if(state & ACCELS_FILMSTRIP)
+      gtk_window_add_accel_group(
+          GTK_WINDOW(darktable.gui->widgets.main_window),
+          darktable.control->accels_filmstrip);
+  }
+}
+
+static void state_from_accels(gpointer data, gpointer state)
+{
+  guint* s = (guint*)state;
+  if(data == (gpointer)darktable.control->accels_global)
+    *s |= ACCELS_GLOBAL;
+  else if(data == (gpointer)darktable.control->accels_lighttable)
+    *s |= ACCELS_LIGHTTABLE;
+  else if(data == (gpointer)darktable.control->accels_darkroom)
+    *s |= ACCELS_DARKROOM;
+  else if(data == (gpointer)darktable.control->accels_capture)
+    *s |= ACCELS_CAPTURE;
+  else if(data == (gpointer)darktable.control->accels_filmstrip)
+    *s |= ACCELS_FILMSTRIP;
+
+  gtk_window_remove_accel_group(
+      GTK_WINDOW(darktable.gui->widgets.main_window),
+      (GtkAccelGroup*)data);
 }
 
 void dt_control_key_accelerators_off(struct dt_control_t *s)
 {
+  GSList *g = gtk_accel_groups_from_object(
+      G_OBJECT(darktable.gui->widgets.main_window));
+  guint state = 0;
+
+  // Setting the apropriate bits for currently active accel groups
+  g_slist_foreach(g, state_from_accels, &state);
+
   s->key_accelerators_on = 0;
+  s->key_accelerators_saved = state;
 }
 
 
@@ -1271,93 +1336,46 @@ void dt_control_save_gui_settings(dt_ctl_gui_mode_t mode)
   dt_conf_set_int("ui_last/expander_metadata", bit);
 }
 
-int dt_control_key_pressed_override(uint16_t which)
+int dt_control_key_pressed_override(guint key, guint state)
 {
-  int fullscreen, visible;
+  int visible;
   GtkWidget *widget;
+  dt_control_accels_t* accels = &darktable.control->accels;
+
   /* check if key accelerators are enabled*/
   if (darktable.control->key_accelerators_on != 1) return 0;
 
-  switch (which)
+  if(key == accels->global_sideborders.accel_key
+     && state == accels->global_sideborders.accel_mods)
   {
-    case KEYCODE_F7:
-      dt_gui_contrast_decrease ();
-      break;
-    case KEYCODE_F8:
-      dt_gui_contrast_increase ();
-      break;
-    case KEYCODE_F9:
-      dt_gui_brightness_decrease();
-      break;
-    case KEYCODE_F10:
-      dt_gui_brightness_increase();
-      break;
+    widget = darktable.gui->widgets.left;
+    visible = GTK_WIDGET_VISIBLE(widget);
+    if(visible) gtk_widget_hide(widget);
+    else gtk_widget_show(widget);
 
-    case KEYCODE_F11:
-      widget = darktable.gui->widgets.main_window;
-      fullscreen = dt_conf_get_bool("ui_last/fullscreen");
-      if(fullscreen) gtk_window_unfullscreen(GTK_WINDOW(widget));
-      else           gtk_window_fullscreen  (GTK_WINDOW(widget));
-      fullscreen ^= 1;
-      dt_conf_set_bool("ui_last/fullscreen", fullscreen);
-      dt_dev_invalidate(darktable.develop);
-      break;
-    case KEYCODE_Escape:
-    case KEYCODE_Caps:
-      widget = darktable.gui->widgets.main_window;
-      gtk_window_unfullscreen(GTK_WINDOW(widget));
-      fullscreen = 0;
-      dt_conf_set_bool("ui_last/fullscreen", fullscreen);
-      dt_dev_invalidate(darktable.develop);
-      break;
-    case KEYCODE_Tab:
-      widget = darktable.gui->widgets.left;
-      visible = GTK_WIDGET_VISIBLE(widget);
-      if(visible) gtk_widget_hide(widget);
-      else gtk_widget_show(widget);
+    widget = darktable.gui->widgets.right;
+    if(visible) gtk_widget_hide(widget);
+    else gtk_widget_show(widget);
 
-      widget = darktable.gui->widgets.right;
-      if(visible) gtk_widget_hide(widget);
-      else gtk_widget_show(widget);
+    dt_dev_invalidate(darktable.develop);
 
-      /*widget = darktable.gui->widgets.bottom;
-      if(visible) gtk_widget_hide(widget);
-      else gtk_widget_show(widget);
-
-      widget = darktable.gui->widgets.top;
-      if(visible) gtk_widget_hide(widget);
-      else gtk_widget_show(widget);*/
-      dt_dev_invalidate(darktable.develop);
-      break;
-    default:
-      return 0;
-      break;
+    widget = darktable.gui->widgets.center;
+    gtk_widget_queue_draw(widget);
+    widget = darktable.gui->widgets.navigation;
+    gtk_widget_queue_draw(widget);
+    return 1;
   }
 
-  widget = darktable.gui->widgets.center;
-  gtk_widget_queue_draw(widget);
-  widget = darktable.gui->widgets.navigation;
-  gtk_widget_queue_draw(widget);
-  return 1;
+  return 0;
 }
 
-int dt_control_key_pressed(uint16_t which)
+int dt_control_key_pressed(guint key, guint state)
 {
-  // this line is here to find the right key code on different platforms (mac).
-  // printf("key code pressed: %d\n", which);
+  int needRedraw;
   GtkWidget *widget;
-  int needRedraw=0;
-  switch (which)
-  {
-    case KEYCODE_period:
-      dt_ctl_switch_mode();
-      needRedraw=1;
-      break;
-    default:
-      // propagate to view modules.
-      needRedraw = dt_view_manager_key_pressed(darktable.view_manager, which);
-      break;
-  }
+
+  needRedraw = dt_view_manager_key_pressed(darktable.view_manager, key,
+                                               state);
   if( needRedraw )
   {
     widget = darktable.gui->widgets.center;
@@ -1368,16 +1386,16 @@ int dt_control_key_pressed(uint16_t which)
   return 1;
 }
 
-int dt_control_key_released(uint16_t which)
+int dt_control_key_released(guint key, guint state)
 {
   // this line is here to find the right key code on different platforms (mac).
   // printf("key code pressed: %d\n", which);
   GtkWidget *widget;
-  switch (which)
+  switch (key)
   {
     default:
       // propagate to view modules.
-      dt_view_manager_key_released(darktable.view_manager, which);
+      dt_view_manager_key_released(darktable.view_manager, key, state);
       break;
   }
 
