@@ -22,6 +22,7 @@
 #include "develop/imageop.h"
 #include "develop/develop.h"
 #include "develop/blend.h"
+#include "develop/tiling.h"
 #include "gui/gtk.h"
 #include "gui/iop_modulegroups.h"
 #include "gui/presets.h"
@@ -227,6 +228,7 @@ int dt_iop_load_module_so(dt_iop_module_so_t *module, const char *libname, const
   if(!g_module_symbol(module->module, "operation_tags",         (gpointer)&(module->operation_tags)))         module->operation_tags = _default_operation_tags;
   if(!g_module_symbol(module->module, "operation_tags_filter",  (gpointer)&(module->operation_tags_filter)))  module->operation_tags_filter = _default_operation_tags_filter;
   if(!g_module_symbol(module->module, "output_bpp",             (gpointer)&(module->output_bpp)))             module->output_bpp = _default_output_bpp;
+  if(!g_module_symbol(module->module, "tiling_callback",        (gpointer)&(module->tiling_callback)))        module->tiling_callback = default_tiling_callback;
   if(!g_module_symbol(module->module, "gui_update",             (gpointer)&(module->gui_update)))             goto error;
   if(!g_module_symbol(module->module, "gui_init",               (gpointer)&(module->gui_init)))               goto error;
   if(!g_module_symbol(module->module, "gui_cleanup",            (gpointer)&(module->gui_cleanup)))            goto error;
@@ -250,8 +252,10 @@ int dt_iop_load_module_so(dt_iop_module_so_t *module, const char *libname, const
   if(!g_module_symbol(module->module, "init_pipe",              (gpointer)&(module->init_pipe)))              goto error;
   if(!g_module_symbol(module->module, "cleanup_pipe",           (gpointer)&(module->cleanup_pipe)))           goto error;
   if(!g_module_symbol(module->module, "process",                (gpointer)&(module->process)))                goto error;
+  if(!g_module_symbol(module->module, "process_tiling",         (gpointer)&(module->process_tiling)))         module->process_tiling = default_process_tiling;
   if(!darktable.opencl->inited ||
       !g_module_symbol(module->module, "process_cl",             (gpointer)&(module->process_cl)))             module->process_cl = NULL;
+  if(!g_module_symbol(module->module, "process_tiling_cl",      (gpointer)&(module->process_tiling_cl)))      module->process_tiling_cl = darktable.opencl->inited ? default_process_tiling_cl : NULL;
   if(!g_module_symbol(module->module, "modify_roi_in",          (gpointer)&(module->modify_roi_in)))          module->modify_roi_in = dt_iop_modify_roi_in;
   if(!g_module_symbol(module->module, "modify_roi_out",         (gpointer)&(module->modify_roi_out)))         module->modify_roi_out = dt_iop_modify_roi_out;
   if(!g_module_symbol(module->module, "legacy_params",          (gpointer)&(module->legacy_params)))          module->legacy_params = NULL;
@@ -297,6 +301,7 @@ dt_iop_load_module_by_so(dt_iop_module_t *module, dt_iop_module_so_t *so, dt_dev
   module->operation_tags  = so->operation_tags;
   module->operation_tags_filter  = so->operation_tags_filter;
   module->output_bpp  = so->output_bpp;
+  module->tiling_callback = so->tiling_callback;
   module->gui_update  = so->gui_update;
   module->gui_init    = so->gui_init;
   module->gui_cleanup = so->gui_cleanup;
@@ -316,7 +321,9 @@ dt_iop_load_module_by_so(dt_iop_module_t *module, dt_iop_module_so_t *so, dt_dev
   module->init_pipe       = so->init_pipe;
   module->cleanup_pipe    = so->cleanup_pipe;
   module->process         = so->process;
+  module->process_tiling  = so->process_tiling;
   module->process_cl      = so->process_cl;
+  module->process_tiling_cl = so->process_tiling_cl;
   module->modify_roi_in   = so->modify_roi_in;
   module->modify_roi_out  = so->modify_roi_out;
   module->legacy_params   = so->legacy_params;
@@ -441,9 +448,10 @@ void dt_iop_load_modules_so()
   darktable.iop = NULL;
   char plugindir[1024], op[20], accelpath[1024];
   const gchar *d_name;
-  dt_get_plugindir(plugindir, 1024);
+  dt_util_get_plugindir(plugindir, 1024);
   g_strlcat(plugindir, "/plugins", 1024);
   GDir *dir = g_dir_open(plugindir, 0, NULL);
+  char name[1024];
   if(!dir) return;
   while((d_name = g_dir_read_name(dir)))
   {
@@ -467,12 +475,26 @@ void dt_iop_load_modules_so()
     if(module->init_key_accels)
       (module->init_key_accels)();
 
+    snprintf(name, 1024, "<Darktable>/darkroom/plugins/%s/reset plugin parameters",module->op);
+    dtgtk_button_init_accel(darktable.control->accels_darkroom,name);
+    snprintf(name, 1024, "<Darktable>/darkroom/plugins/%s/show preset menu",module->op);
+    dtgtk_button_init_accel(darktable.control->accels_darkroom,name);
+    if (module->flags()&IOP_FLAGS_SUPPORTS_BLENDING)
+    {
+	    snprintf(name, 1024, "<Darktable>/darkroom/plugins/%s/fusion opacity",module->op);
+	    dtgtk_slider_init_accel(darktable.control->accels_darkroom,name);
+    }
     if(!(module->flags() & IOP_FLAGS_DEPRECATED))
     {
       // Adding the optional show accelerator to the table (blank)
       snprintf(accelpath, 256, "<Darktable>/darkroom/plugins/%s/show plugin",
                (module->op));
       gtk_accel_map_add_entry(accelpath, 0, 0);
+      dt_accel_group_connect_by_path(darktable.control->accels_darkroom, accelpath,
+                                     NULL);
+      snprintf(accelpath, 256, "<Darktable>/darkroom/plugins/%s/enable plugin",
+               (module->op));
+      gtk_accel_map_add_entry(accelpath, 0, 0);
       dt_accel_group_connect_by_path(darktable.control->accels_darkroom, accelpath,
                                      NULL);
       snprintf(accelpath, 1024, "<Darktable>/darkroom/plugins/%s/reset plugin parameters",module->op);
@@ -880,6 +902,8 @@ GtkWidget *dt_iop_gui_get_expander(dt_iop_module_t *module)
     GtkWidget *label = gtk_label_new(_("mode"));
     bd->blend_modes_combo = GTK_COMBO_BOX(gtk_combo_box_new_text());
     bd->opacity_slider = GTK_WIDGET(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 100.0, 1, 100.0, 0));
+    snprintf(name, 1024, "<Darktable>/darkroom/plugins/%s/fusion opacity",module->op);
+    dtgtk_slider_set_accel(DTGTK_SLIDER(bd->opacity_slider),darktable.control->accels_darkroom,name);
     dtgtk_slider_set_label(DTGTK_SLIDER(bd->opacity_slider),_("opacity"));
     dtgtk_slider_set_unit(DTGTK_SLIDER(bd->opacity_slider),"%");
     gtk_combo_box_append_text(GTK_COMBO_BOX(bd->blend_modes_combo), _("normal"));
@@ -1041,7 +1065,10 @@ dt_iop_clip_and_zoom(float *out, const float *const in,
           num++;
         }
       // col = _mm_mul_ps(col, _mm_set1_ps(1.0f/((2.0f*samples+1.0f)*(2.0f*samples+1.0f))));
-      col = _mm_mul_ps(col, _mm_set1_ps(1.0f/num));
+      if(num == 0.0f)
+        col = _mm_load_ps(in + 4*(CLAMPS(px, 0, roi_in->width-1) + in_stride*CLAMPS(py, 0, roi_in->height-1)));
+      else
+        col = _mm_mul_ps(col, _mm_set1_ps(1.0f/num));
       // memcpy(outc, &col, 4*sizeof(float));
       _mm_stream_ps(outc, col);
       outc += 4;
@@ -1056,22 +1083,6 @@ FC(const int row, const int col, const unsigned int filters)
   return filters >> ((((row) << 1 & 14) + ((col) & 1)) << 1) & 3;
 }
 
-#if 0
-static float
-weight (const float c1, const float c2)
-{
-  // return dt_fast_expf(-(c1-c2)*(c1-c2)*(1.0f/(0.001f*65535.0f*65535.0f)));
-  return ((c1 > 65500.0f) ^ (c2 > 65500.0f)) ? 0.0f : 1.0f;
-}
-#else
-/*static float
-weight (const float c0, const float c1, const float c2, const float c3, const float c4)
-{
-  const float c = 65534.0f;
-  const float cc = fmaxf(fmaxf(c1, c2), fmaxf(c3, c4));
-  return ((c0 > c) ^ (cc > c)) ? 0.001f : 1.0f;
-}*/
-#endif
 /**
  * downscales and clips a mosaiced buffer (in) to the given region of interest (r_*)
  * and writes it to out in float4 format.
@@ -1082,24 +1093,18 @@ void
 dt_iop_clip_and_zoom_demosaic_half_size(float *out, const uint16_t *const in,
                                         const dt_iop_roi_t *const roi_out, const dt_iop_roi_t * const roi_in, const int32_t out_stride, const int32_t in_stride, const unsigned int filters)
 {
+#if 0
+  printf("scale: %f\n",roi_out->scale);
+  struct timeval tm1,tm2;
+  gettimeofday(&tm1,NULL);
+  for (int k = 0 ; k < 100 ; k++)
+  {
+#endif
   // adjust to pixel region and don't sample more than scale/2 nbs!
   // pixel footprint on input buffer, radius:
-  const float px_footprint = .9f/roi_out->scale;
+  const float px_footprint = 1.f/roi_out->scale;
   // how many 2x2 blocks can be sampled inside that area
-  const int samples = ((int)px_footprint)/2;
-
-  // init gauss with sigma = samples (half footprint)
-  float filter[2*samples + 1];
-  float sum = 0.0f;
-
-  memset(filter, 0, sizeof(float)*(2*samples + 1));
-
-  if(samples)
-  {
-    for(int i=-samples; i<=samples; i++) sum += (filter[i+samples] = expf(-i*i/(float)(.5f*samples*samples)));
-    for(int k=0; k<2*samples+1; k++) filter[k] /= sum;
-  }
-  else filter[0] = 1.0f;
+  const int samples = round(px_footprint/2);
 
   // move p to point to an rggb block:
   int trggbx = 0, trggby = 0;
@@ -1110,71 +1115,67 @@ dt_iop_clip_and_zoom_demosaic_half_size(float *out, const uint16_t *const in,
     trggby ++;
   }
   const int rggbx = trggbx, rggby = trggby;
-  const int maxj = ((roi_in->height-3)&~1u)+rggby;
-  const int maxi = ((roi_in->width -3)&~1u)+rggbx;
 
-  const __m128 ff  = _mm_set1_ps(1.0f/65535.0f);
-  // const __m128 one = _mm_set1_ps(1.0f);
 #ifdef _OPENMP
-  #pragma omp parallel for default(none) shared(out, filter) schedule(static)
+  #pragma omp parallel for default(none) shared(out) schedule(static)
 #endif
   for(int y=0; y<roi_out->height; y++)
   {
-    int py = (y + roi_out->y)/roi_out->scale;
-    py = MAX(0, py & ~1) + rggby;
-    py = MIN((((roi_in->height-3) & ~1u) + rggby), py);
-    const int maxjj = MIN(maxj,  py + 2*samples);
-    const int minjj = MAX(rggby, py - 2*samples);
+    float *outc = out + 4*(out_stride*y);
 
-    float *outc = out + 4*out_stride*y;
+    float fy = (y + roi_out->y)*px_footprint;
+    int py = (int)fy & ~1;
+    py = MIN(((roi_in->height-4) & ~1u), py) + rggby;
+
+    int maxj = MIN(((roi_in->height-3)&~1u)+rggby, py+2*samples);
+
+    float fx = roi_out->x*px_footprint;
+      
     for(int x=0; x<roi_out->width; x++)
     {
       __m128 col = _mm_setzero_ps();
-      // _mm_prefetch
-      int px = (x + roi_out->x)/roi_out->scale;
 
-      // round down to next even number and jump to rggb block:
-      px = MAX(0, px & ~1) + rggbx;
-      px = MIN((((roi_in->width -3) & ~1u) + rggbx), px);
+      fx += px_footprint;
+      int px = (int)fx & ~1;
+      px = MIN(((roi_in->width -4) & ~1u), px) + rggbx;
 
-      const int idx = px   + in_stride*py;
+      int maxi = MIN(((roi_in->width -3)&~1u)+rggbx, px+2*samples);
 
-      // const float pc1  = in[px   + in_stride*py];
-      // const float pc23 = .5f*(in[px+1 + in_stride*py] + in[px   + in_stride*(py + 1)]);
-      // const float pc4  = in[px+1 + in_stride*(py + 1)];
+      int num = 0;
+
+      const int idx = px + in_stride*py;
       const uint16_t pc = MAX(MAX(in[idx], in[idx+1]), MAX(in[idx + in_stride], in[idx+1 + in_stride]));
 
-      __m128 num = _mm_setzero_ps();
-      const int maxii = MIN(maxi,  px + 2*samples);
-      const int minii = MAX(rggbx, px - 2*samples);
-      for(int j=minjj; j<=maxjj; j+=2)
-        for(int i=minii; i<=maxii; i+=2)
+      // 2x2 blocks in the middle of sampling region
+      __m128i sum = _mm_set_epi32(0,0,0,0);
+
+      for(int j=py; j<=maxj; j+=2)
+        for(int i=px; i<=maxi; i+=2)
         {
-          // get four mosaic pattern uint16:
           const uint16_t p1 = in[i   + in_stride*j];
           const uint16_t p2 = in[i+1 + in_stride*j];
           const uint16_t p3 = in[i   + in_stride*(j + 1)];
           const uint16_t p4 = in[i+1 + in_stride*(j + 1)];
 
-          // const float wr = weight(pc1, p1);
-          // const float wg = weight(pc23, .5f*(p2+p3));
-          const float w = ((pc == 65535) ^ (MAX(MAX(p1,p2),MAX(p3,p4)) == 65535)) ? 0.001f : 1.0f;
-          // const float wb = weight(pc4, p4);
-
-          // const float f = filter[(i-px)/2+samples]*filter[(j-py)/2+samples];
-          // col = _mm_add_ps(col, _mm_mul_ps(_mm_set_ps(0.0f, p4*wb, .5f*(p2+p3)*wg, p1*wr), _mm_set1_ps(f/65535.0f)));
-          // num = _mm_add_ps(num, _mm_mul_ps(_mm_set_ps(1.0f, wb, wg, wr), _mm_set1_ps(f)));
-          // col = _mm_add_ps(col, _mm_mul_ps(_mm_set_ps(0.0f, p4*wb, .5f*(p2+p3)*wg, p1*wr), ff));
-          // num = _mm_add_ps(num, _mm_mul_ps(_mm_set_ps(1.0f, wb, wg, wr), one));
-          col = _mm_add_ps(col, _mm_mul_ps(_mm_set_ps(0.0f, p4*w, .5f*(p2+p3)*w, p1*w), ff));
-          num = _mm_add_ps(num, _mm_set1_ps(w));
+          if (!((pc == 65535) ^ (MAX(MAX(p1,p2),MAX(p3,p4)) == 65535)))
+          {
+            sum = _mm_add_epi32(sum, _mm_set_epi32(0,p4,p3+p2,p1));
+            num++;
+          }
         }
-      col = _mm_div_ps(col, num);
+
+      col = _mm_mul_ps(_mm_cvtepi32_ps(sum), _mm_div_ps(_mm_set_ps(0.0f,1.0f/65535.0f,0.5f/65535.0f,1.0f/65535.0f),_mm_set1_ps(num)));
       _mm_stream_ps(outc, col);
       outc += 4;
     }
   }
   _mm_sfence();
+#if 0
+  }
+  gettimeofday(&tm2,NULL);
+  float perf = (tm2.tv_sec-tm1.tv_sec)*1000.0f + (tm2.tv_usec-tm1.tv_usec)/1000.0f;
+  printf("time spent: %.4f\n",perf/100.0f);
+#endif
 }
 
 void
@@ -1183,22 +1184,9 @@ dt_iop_clip_and_zoom_demosaic_half_size_f(float *out, const float *const in,
 {
   // adjust to pixel region and don't sample more than scale/2 nbs!
   // pixel footprint on input buffer, radius:
-  const float px_footprint = .9f/roi_out->scale;
+  const float px_footprint = 1.f/roi_out->scale;
   // how many 2x2 blocks can be sampled inside that area
-  const int samples = ((int)px_footprint)/2;
-
-  // init gauss with sigma = samples (half footprint)
-  float filter[2*samples + 1];
-  float sum = 0.0f;
-
-  memset(filter, 0, sizeof(float)*(2*samples + 1));
-
-  if(samples)
-  {
-    for(int i=-samples; i<=samples; i++) sum += (filter[i+samples] = expf(-i*i/(float)(.5f*samples*samples)));
-    for(int k=0; k<2*samples+1; k++) filter[k] /= sum;
-  }
-  else filter[0] = 1.0f;
+  const int samples = round(px_footprint/2);
 
   // move p to point to an rggb block:
   int trggbx = 0, trggby = 0;
@@ -1210,64 +1198,156 @@ dt_iop_clip_and_zoom_demosaic_half_size_f(float *out, const float *const in,
   }
   const int rggbx = trggbx, rggby = trggby;
 
-
 #ifdef _OPENMP
-  #pragma omp parallel for default(none) shared(out, filter) schedule(static)
+  #pragma omp parallel for default(none) shared(out) schedule(static)
 #endif
   for(int y=0; y<roi_out->height; y++)
   {
     float *outc = out + 4*(out_stride*y);
+
+    float fy = (y + roi_out->y)*px_footprint;
+    int py = (int)fy & ~1;
+    const float dy = (fy - py)/2;
+    py = MIN(((roi_in->height-6) & ~1u), py) + rggby;
+
+    int maxj = MIN(((roi_in->height-5)&~1u)+rggby, py+2*samples);
+
+    float fx = roi_out->x*px_footprint;
+      
     for(int x=0; x<roi_out->width; x++)
     {
       __m128 col = _mm_setzero_ps();
-      // _mm_prefetch
-      // upper left corner:
-      float fx = (x + roi_out->x)/roi_out->scale, fy = (y + roi_out->y)/roi_out->scale;
-      int px = (int)fx, py = (int)fy;
 
-      // round down to next even number and jump to rggb block:
-      px = MAX(0, px & ~1) + rggbx;
-      py = MAX(0, py & ~1) + rggby;
+      fx += px_footprint;
+      int px = (int)fx & ~1;
+      const float dx = (fx - px)/2;
+      px = MIN(((roi_in->width -6) & ~1u), px) + rggbx;
 
-      px = MIN((((roi_in->width -5) & ~1u) + rggbx),  px);
-      py = MIN((((roi_in->height-5) & ~1u) + rggby), py);
+      int maxi = MIN(((roi_in->width -5)&~1u)+rggbx, px+2*samples);
 
-      const float dx = .5f*(fx - px), dy = .5f*(fy - py);
-      const __m128 d0 = _mm_set1_ps((1.0f-dx)*(1.0f-dy));
-      const __m128 d1 = _mm_set1_ps((dx)*(1.0f-dy));
-      const __m128 d2 = _mm_set1_ps((1.0f-dx)*(dy));
-      const __m128 d3 = _mm_set1_ps(dx*dy);
+      float p1, p2, p4;
+      int i,j;
+      float num = 0;
 
-      float num = 0.0f;
-      for(int j=MAX(rggby, py-2*samples); j<=MIN(((roi_in->height-5)&~1u)+rggby, py+2*samples); j+=2)
-        for(int i=MAX(rggbx, px-2*samples); i<=MIN(((roi_in->width -5)&~1u)+rggbx, px+2*samples); i+=2)
+      // upper left 2x2 block of sampling region
+      p1 = in[px   + in_stride*py];
+      p2 = in[px+1 + in_stride*py] + in[px   + in_stride*(py + 1)];
+      p4 = in[px+1 + in_stride*(py + 1)];
+      col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps((1-dx)*(1-dy)),_mm_set_ps(0.0f, p4, p2, p1)));
+
+      // left 2x2 block border of sampling region
+      for (j = py+2 ; j <= maxj ; j+=2)
+      {
+          p1 = in[px   + in_stride*j];
+          p2 = in[px+1 + in_stride*j] + in[px   + in_stride*(j + 1)];
+          p4 = in[px+1 + in_stride*(j + 1)];
+          col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(1-dx),_mm_set_ps(0.0f, p4, p2, p1)));
+      }
+
+      // upper 2x2 block border of sampling region
+      for (i = px+2 ; i <= maxi ; i+=2)
+      {
+          p1 = in[i   + in_stride*py];
+          p2 = in[i+1 + in_stride*py] + in[i   + in_stride*(py + 1)];
+          p4 = in[i+1 + in_stride*(py + 1)];
+          col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(1-dy),_mm_set_ps(0.0f, p4, p2, p1)));
+      }
+
+      // 2x2 blocks in the middle of sampling region
+      for(int j=py+2; j<=maxj; j+=2)
+        for(int i=px+2; i<=maxi; i+=2)
         {
-          // get four mosaic pattern uint16:
-          float p1, p2, p4;
           p1 = in[i   + in_stride*j];
           p2 = in[i+1 + in_stride*j] + in[i   + in_stride*(j + 1)];
           p4 = in[i+1 + in_stride*(j + 1)];
-          const __m128 px0 = _mm_set_ps(0.0f, p4, .5f*p2, p1);
-          p1 = in[i+2   + in_stride*j];
-          p2 = in[i+2+1 + in_stride*j] + in[i+2   + in_stride*(j + 1)];
-          p4 = in[i+2+1 + in_stride*(j + 1)];
-          const __m128 px1 = _mm_set_ps(0.0f, p4, .5f*p2, p1);
-          p1 = in[i   + in_stride*(j+2)];
-          p2 = in[i+1 + in_stride*(j+2)] + in[i   + in_stride*(j+2 + 1)];
-          p4 = in[i+1 + in_stride*(j+2 + 1)];
-          const __m128 px2 = _mm_set_ps(0.0f, p4, .5f*p2, p1);
-          p1 = in[i+2   + in_stride*(j+2)];
-          p2 = in[i+2+1 + in_stride*(j+2)] + in[i+2   + in_stride*(j+2 + 1)];
-          p4 = in[i+2+1 + in_stride*(j+2 + 1)];
-          const __m128 px3 = _mm_set_ps(0.0f, p4, .5f*p2, p1);
-          const __m128 lerp = _mm_add_ps(_mm_add_ps(_mm_mul_ps(d0, px0), _mm_mul_ps(d1, px1)),
-                                         _mm_add_ps(_mm_mul_ps(d2, px2), _mm_mul_ps(d3, px3)));
-
-          const float f = filter[(i-px)/2+samples]*filter[(j-py)/2+samples];
-          col = _mm_add_ps(col, _mm_mul_ps(lerp, _mm_set1_ps(f)));
-          num += f;
+          col = _mm_add_ps(col, _mm_set_ps(0.0f, p4, p2, p1));
         }
-      col = _mm_mul_ps(col, _mm_set1_ps(1.0f/num));
+
+      if (maxi == px + 2*samples && maxj == py + 2*samples)
+      {
+        // right border
+        for (j = py+2 ; j <= maxj ; j+=2)
+        {
+          p1 = in[maxi+2   + in_stride*j];
+          p2 = in[maxi+3 + in_stride*j] + in[maxi+2   + in_stride*(j + 1)];
+          p4 = in[maxi+3 + in_stride*(j + 1)];
+          col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dx),_mm_set_ps(0.0f, p4, p2, p1)));
+        }
+
+        // upper right
+        p1 = in[maxi+2   + in_stride*py];
+        p2 = in[maxi+3 + in_stride*py] + in[maxi+2   + in_stride*(py + 1)];
+        p4 = in[maxi+3 + in_stride*(py + 1)];
+        col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dx*(1-dy)),_mm_set_ps(0.0f, p4, p2, p1)));
+
+        // lower border
+        for (i = px+2 ; i <= maxi ; i+=2)
+        {
+          p1 = in[i   + in_stride*(maxj+2)];
+          p2 = in[i+1 + in_stride*(maxj+2)] + in[i   + in_stride*(maxj+3)];
+          p4 = in[i+1 + in_stride*(maxj+3)];
+          col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dy),_mm_set_ps(0.0f, p4, p2, p1)));
+        }
+
+        // lower left 2x2 block
+        p1 = in[px   + in_stride*(maxj+2)];
+        p2 = in[px+1 + in_stride*(maxj+2)] + in[px   + in_stride*(maxj+3)];
+        p4 = in[px+1 + in_stride*(maxj+3)];
+        col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps((1-dx)*dy),_mm_set_ps(0.0f, p4, p2, p1)));
+
+        // lower right 2x2 block
+        p1 = in[maxi+2   + in_stride*(maxj+2)];
+        p2 = in[maxi+3 + in_stride*(maxj+2)] + in[maxi   + in_stride*(maxj+3)];
+        p4 = in[maxi+3 + in_stride*(maxj+3)];
+        col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dx*dy),_mm_set_ps(0.0f, p4, p2, p1)));
+
+        num = (samples+1)*(samples+1);
+      }
+      else if (maxi == px + 2*samples)
+      {
+        // right border
+        for (j = py+2 ; j <= maxj ; j+=2)
+        {
+          p1 = in[maxi+2   + in_stride*j];
+          p2 = in[maxi+3 + in_stride*j] + in[maxi+2   + in_stride*(j + 1)];
+          p4 = in[maxi+3 + in_stride*(j + 1)];
+          col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dx),_mm_set_ps(0.0f, p4, p2, p1)));
+        }
+
+        // upper right
+        p1 = in[maxi+2   + in_stride*py];
+        p2 = in[maxi+3 + in_stride*py] + in[maxi+2   + in_stride*(py + 1)];
+        p4 = in[maxi+3 + in_stride*(py + 1)];
+        col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dx*(1-dy)),_mm_set_ps(0.0f, p4, p2, p1)));
+
+        num = ((maxj-py)/2+1-dy)*(samples+1);
+      }
+      else if (maxj == py + 2*samples)
+      {
+        // lower border
+        for (i = px+2 ; i <= maxi ; i+=2)
+        {
+          p1 = in[i   + in_stride*(maxj+2)];
+          p2 = in[i+1 + in_stride*(maxj+2)] + in[i   + in_stride*(maxj+3)];
+          p4 = in[i+1 + in_stride*(maxj+3)];
+          col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps(dy),_mm_set_ps(0.0f, p4, p2, p1)));
+        }
+
+        // lower left 2x2 block
+        p1 = in[px   + in_stride*(maxj+2)];
+        p2 = in[px+1 + in_stride*(maxj+2)] + in[px   + in_stride*(maxj+3)];
+        p4 = in[px+1 + in_stride*(maxj+3)];
+        col = _mm_add_ps(col, _mm_mul_ps(_mm_set1_ps((1-dx)*dy),_mm_set_ps(0.0f, p4, p2, p1)));
+
+        num = ((maxi-px)/2+1-dx)*(samples+1);
+      }
+      else
+      {
+        num = ((maxi-px)/2+1-dx)*((maxj-py)/2+1-dy);
+      }
+
+      num = 1.0f/num;
+      col = _mm_mul_ps(col, _mm_set_ps(0.0f,num,0.5f*num,num));
       _mm_stream_ps(outc, col);
       outc += 4;
     }
