@@ -241,7 +241,7 @@ static void tree_insert_rec(GtkTreeStore *model, GtkTreeIter *parent,
     gtk_tree_store_set(model, &iter,
                        ACCEL_COLUMN, accel_path,
                        BINDING_COLUMN, g_dpgettext2("gtk20", "keyboard label", name),
-                       TRANS_COLUMN, _(accel_path),
+                       TRANS_COLUMN, g_dpgettext2(NULL, "accel", accel_path),
                        -1);
     g_free(name);    
   } 
@@ -305,7 +305,7 @@ static void tree_insert_rec(GtkTreeStore *model, GtkTreeIter *parent,
 
       /* not an iop or lib, do a translation.. */
       if(!translation)
-        translation = _(node);
+        translation = g_dpgettext2(NULL, "accel",node);
       
       gtk_tree_store_append(model, &iter, parent);
       gtk_tree_store_set(model, &iter,
@@ -514,7 +514,7 @@ static gboolean tree_key_press(GtkWidget *widget, GdkEventKey *event,
   if(event->is_modifier)
     return FALSE;
 
-  dt_get_user_config_dir(datadir, 1024);
+  dt_util_get_user_config_dir(datadir, 1024);
   snprintf(accelpath, 1024, "%s/keyboardrc", datadir);
 
   // Otherwise, determine whether we're in remap mode or not
@@ -693,7 +693,7 @@ static void import_export(GtkButton *button, gpointer data)
             gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser)));
 
         // Saving to the permanent keyboardrc
-        dt_get_user_config_dir(confdir, 1024);
+        dt_util_get_user_config_dir(confdir, 1024);
         snprintf(accelpath, 1024, "%s/keyboardrc", confdir);
         gtk_accel_map_save(accelpath);
 
@@ -723,7 +723,7 @@ static void restore_defaults(GtkButton *button, gpointer data)
   if(gtk_dialog_run(GTK_DIALOG(message)) == GTK_RESPONSE_OK)
   {
     // First load the default keybindings for immediate effect
-    dt_get_user_config_dir(dir, 1024);
+    dt_util_get_user_config_dir(dir, 1024);
     snprintf(path, 1024, "%s/keyboardrc_default", dir);
     gtk_accel_map_load(path);
 
@@ -739,7 +739,7 @@ static void restore_defaults(GtkButton *button, gpointer data)
 
     // Then delete any changes to the user's keyboardrc so it gets reset
     // on next startup
-    dt_get_user_config_dir(dir, 1024);
+    dt_util_get_user_config_dir(dir, 1024);
     snprintf(path, 1024, "%s/keyboardrc", dir);
     g_file_delete(g_file_new_for_path(path), NULL, NULL);
   }
@@ -781,7 +781,7 @@ static gint compare_rows(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
   // Otherwise just return alphabetical order
   gtk_tree_model_get(model, a, TRANS_COLUMN, &a_text, -1);
   gtk_tree_model_get(model, b, TRANS_COLUMN, &b_text, -1);
-  return strcmp(a_text, b_text);
+  return strcasecmp(a_text, b_text);
 
 }
 
