@@ -270,7 +270,6 @@ flickcurl_upload_status static *_flickr_api_upload_photo( dt_storage_flickr_para
   params->title = caption;
   params->description = description;
 
-  //TODO: handle tags with two or more words
   if (tags)
   {
 
@@ -289,7 +288,7 @@ flickcurl_upload_status static *_flickr_api_upload_photo( dt_storage_flickr_para
         dt_tag_t *t = g_list_nth_data(tags,i);
         if (t)
         {
-          array[i] = g_strdup (t->tag);
+          array[i] = g_strconcat ("\"", t->tag, "\"", NULL);
         }
 
       }
@@ -648,7 +647,7 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
 
   /* construct a temporary file name */
   char fname[4096]= {0};
-  dt_get_user_local_dir (fname,4096);
+  dt_util_get_user_local_dir (fname,4096);
   g_strlcat (fname,"/tmp",4096);
   g_mkdir_with_parents(fname,0700);
   g_strlcat (fname,"/darktable.XXXXXX.",4096);
