@@ -33,6 +33,8 @@
 #include "dtgtk/resetlabel.h"
 #include "external/adobe_coeff.c"
 
+#define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
+
 DT_MODULE(1)
 
 typedef struct dt_iop_colorin_global_data_t
@@ -54,6 +56,13 @@ groups ()
   return IOP_GROUP_COLOR;
 }
 
+int
+flags ()
+{
+  return IOP_FLAGS_ALLOW_TILING;
+}
+
+
 void
 init_global(dt_iop_module_so_t *module)
 {
@@ -132,7 +141,10 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   cl_int err = -999;
   const int map_blues = self->dev->image->flags & DT_IMAGE_RAW;
   const int devid = piece->pipe->devid;
-  size_t sizes[] = {roi_in->width, roi_in->height, 1};
+  const int width = roi_in->width;
+  const int height = roi_in->height;
+
+  size_t sizes[] = { ROUNDUP(width, 4), ROUNDUP(height, 4), 1};
   dev_m = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*9, d->cmatrix);
   if (dev_m == NULL) goto error;
   dev_r = dt_opencl_copy_host_to_device(devid, d->lut[0], 256, 256, sizeof(float));
@@ -145,12 +157,14 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   if (dev_coeffs == NULL) goto error;
   dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 0, sizeof(cl_mem), (void *)&dev_in);
   dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 1, sizeof(cl_mem), (void *)&dev_out);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 2, sizeof(cl_mem), (void *)&dev_m);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 3, sizeof(cl_mem), (void *)&dev_r);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 4, sizeof(cl_mem), (void *)&dev_g);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 5, sizeof(cl_mem), (void *)&dev_b);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 6, sizeof(cl_int), (void *)&map_blues);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 7, sizeof(cl_mem), (void *)&dev_coeffs);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 2, sizeof(int), (void *)&width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 3, sizeof(int), (void *)&height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 4, sizeof(cl_mem), (void *)&dev_m);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 5, sizeof(cl_mem), (void *)&dev_r);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 6, sizeof(cl_mem), (void *)&dev_g);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 7, sizeof(cl_mem), (void *)&dev_b);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 8, sizeof(cl_int), (void *)&map_blues);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_colorin, 9, sizeof(cl_mem), (void *)&dev_coeffs);
   err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_colorin, sizes);
   if(err != CL_SUCCESS) goto error;
   dt_opencl_release_mem_object(dev_m);
@@ -297,7 +311,7 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   piece->process_cl_ready = 1;
   char datadir[1024];
   char filename[1024];
-  dt_get_datadir(datadir, 1024);
+  dt_util_get_datadir(datadir, 1024);
   if(!strcmp(p->iccprofile, "darktable"))
   {
     char makermodel[1024];
@@ -308,14 +322,15 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   if(!strcmp(p->iccprofile, "cmatrix"))
   {
     // color matrix
-    dt_image_full_path(self->dev->image->id, filename, 1024);
     char makermodel[1024];
     dt_colorspaces_get_makermodel(makermodel, 1024, self->dev->image->exif_maker, self->dev->image->exif_model);
     float cam_xyz[12];
+    cam_xyz[0] = -666.0f;
     dt_dcraw_adobe_coeff(makermodel, "", (float (*)[12])cam_xyz);
-    d->input = dt_colorspaces_create_xyzimatrix_profile((float (*)[3])cam_xyz);
+    if(cam_xyz[0] == -666.0f) sprintf(p->iccprofile, "linear_rgb");
+    else d->input = dt_colorspaces_create_xyzimatrix_profile((float (*)[3])cam_xyz);
   }
-  else if(!strcmp(p->iccprofile, "sRGB"))
+  if(!strcmp(p->iccprofile, "sRGB"))
   {
     d->input = dt_colorspaces_create_srgb_profile();
   }
@@ -465,7 +480,7 @@ void init(dt_iop_module_t *module)
   module->default_params = malloc(sizeof(dt_iop_colorin_params_t));
   module->params_size = sizeof(dt_iop_colorin_params_t);
   module->gui_data = NULL;
-  module->priority = 333; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 347; // module order created by iop_dependencies.py, do not edit!
   module->hide_enable_button = 1;
 }
 
@@ -485,16 +500,24 @@ void gui_init(struct dt_iop_module_t *self)
   dt_iop_colorin_params_t *p   = (dt_iop_colorin_params_t *)self->params;
 
   g->profiles = NULL;
+  dt_iop_color_profile_t *prof;
+  int pos = -1;
 
   // get color matrix from raw image:
-  dt_iop_color_profile_t *prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
-  g_strlcpy(prof->filename, "cmatrix", sizeof(prof->filename));
-  g_strlcpy(prof->name, "cmatrix", sizeof(prof->name));
-  g->profiles = g_list_append(g->profiles, prof);
-  int pos = prof->pos = 0;
-
   char makermodel[1024];
   dt_colorspaces_get_makermodel(makermodel, 1024, self->dev->image->exif_maker, self->dev->image->exif_model);
+  float cam_xyz[12];
+  cam_xyz[0] = -666.0f;
+  dt_dcraw_adobe_coeff(makermodel, "", (float (*)[12])cam_xyz);
+  if(cam_xyz[0] != -666.0f)
+  {
+    prof = (dt_iop_color_profile_t *)g_malloc0(sizeof(dt_iop_color_profile_t));
+    g_strlcpy(prof->filename, "cmatrix", sizeof(prof->filename));
+    g_strlcpy(prof->name, "cmatrix", sizeof(prof->name));
+    g->profiles = g_list_append(g->profiles, prof);
+    prof->pos = ++pos;
+  }
+
   // darktable built-in, if applicable
   for(int k=0; k<dt_profiled_colormatrix_cnt; k++)
   {
@@ -546,8 +569,8 @@ void gui_init(struct dt_iop_module_t *self)
 
   // read {userconfig,datadir}/color/in/*.icc, in this order.
   char datadir[1024], confdir[1024], dirname[1024], filename[1024];
-  dt_get_user_config_dir(confdir, 1024);
-  dt_get_datadir(datadir, 1024);
+  dt_util_get_user_config_dir(confdir, 1024);
+  dt_util_get_datadir(datadir, 1024);
   snprintf(dirname, 1024, "%s/color/in", confdir);
   if(!g_file_test(dirname, G_FILE_TEST_IS_DIR))
     snprintf(dirname, 1024, "%s/color/in", datadir);
