@@ -24,6 +24,7 @@
 #include <string.h>
 #include "develop/develop.h"
 #include "develop/imageop.h"
+#include "develop/tiling.h"
 #include "control/control.h"
 #include "common/opencl.h"
 #include "dtgtk/slider.h"
@@ -35,6 +36,9 @@
 DT_MODULE(1)
 
 #define MAXR 12
+#define BLOCKSIZE 2048		/* maximum blocksize. must be a power of 2 and will be automatically reduced if needed */
+
+#define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
 
 typedef struct dt_iop_sharpen_params_t
 {
@@ -57,7 +61,9 @@ dt_iop_sharpen_data_t;
 
 typedef struct dt_iop_sharpen_global_data_t
 {
-  int kernel_sharpen;
+  int kernel_sharpen_hblur;
+  int kernel_sharpen_vblur;
+  int kernel_sharpen_mix;
 }
 dt_iop_sharpen_global_data_t;
 
@@ -77,9 +83,18 @@ groups ()
 int
 flags ()
 {
-  return IOP_FLAGS_SUPPORTS_BLENDING;
+  return IOP_FLAGS_SUPPORTS_BLENDING | IOP_FLAGS_ALLOW_TILING;
 }
 
+
+void init_key_accels()
+{
+  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/sharpen/radius");
+  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/sharpen/amount");
+  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/sharpen/threshold");
+}
+
+
 #ifdef HAVE_OPENCL
 int
 process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
@@ -90,36 +105,112 @@ process_cl (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem
   cl_int err = -999;
 
   const int devid = piece->pipe->devid;
+  const int width = roi_in->width;
+  const int height = roi_in->height;
   const int rad = MIN(MAXR, ceilf(d->radius * roi_in->scale / piece->iscale));
+  const int wd = 2*rad+1;
+  float mat[wd];
 
   if(rad == 0)
   {
     size_t origin[] = {0, 0, 0};
-    size_t region[] = {roi_in->width, roi_in->height, 1};
+    size_t region[] = {width, height, 1};
     err = dt_opencl_enqueue_copy_image(devid, dev_in, dev_out, origin, origin, region);
     if (err != CL_SUCCESS) goto error;
     return TRUE;
   }
+
   // init gaussian kernel
-  float mat[2*(MAXR+1)];
-  const int wd = 2*rad+1;
   float *m = mat + rad;
   const float sigma2 = (1.0f/(2.5*2.5))*(d->radius*roi_in->scale/piece->iscale)*(d->radius*roi_in->scale/piece->iscale);
   float weight = 0.0f;
   for(int l=-rad; l<=rad; l++) weight += m[l] = expf(- (l*l)/(2.f*sigma2));
   for(int l=-rad; l<=rad; l++) m[l] /= weight;
-  size_t sizes[] = {roi_in->width, roi_in->height, 1};
+
+
+  // prepare local work group
+  size_t maxsizes[3] = { 0 };        // the maximum dimensions for a work group
+  size_t workgroupsize = 0;          // the maximum number of items in a work group
+  unsigned long localmemsize = 0;    // the maximum amount of local memory we can use
+  
+  // make sure blocksize is not too large
+  size_t blocksize = BLOCKSIZE;
+  if(dt_opencl_get_work_group_limits(devid, maxsizes, &workgroupsize, &localmemsize) == CL_SUCCESS)
+  {
+    // reduce blocksize step by step until it fits to limits
+    while(blocksize > maxsizes[0] || blocksize > maxsizes[1] 
+          || blocksize > workgroupsize || (blocksize+2*rad)*sizeof(float) > localmemsize)
+    {
+      if(blocksize == 1) break;
+      blocksize >>= 1;    
+    }
+  }
+  else
+  {
+    blocksize = 1;   // slow but safe
+  }
+
+  // width and height of intermediate buffers. Need to be multiples of BLOCKSIZE
+  const size_t bwidth = width % blocksize == 0 ? width : (width / blocksize + 1)*blocksize;
+  const size_t bheight = height % blocksize == 0 ? height : (height / blocksize + 1)*blocksize;
+
+  size_t sizes[3];
+  size_t local[3];
+
   dev_m = dt_opencl_copy_host_to_device_constant(devid, sizeof(float)*wd, mat);
   if (dev_m == NULL) goto error;
-  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen, 0, sizeof(cl_mem), (void *)&dev_in);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen, 1, sizeof(cl_mem), (void *)&dev_out);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen, 2, sizeof(cl_mem), (void *)&dev_m);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen, 3, sizeof(int), (void *)&rad);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen, 4, sizeof(float), (void *)&d->amount);
-  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen, 5, sizeof(float), (void *)&d->threshold);
-  err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_sharpen, sizes);
+
+  /* horizontal blur */
+  sizes[0] = bwidth;
+  sizes[1] = ROUNDUP(height, 4);
+  sizes[2] = 1;
+  local[0] = blocksize;
+  local[1] = 1;
+  local[2] = 1;
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 0, sizeof(cl_mem), (void *)&dev_in);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 2, sizeof(cl_mem), (void *)&dev_m);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 3, sizeof(int), (void *)&rad);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 4, sizeof(int), (void *)&width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 5, sizeof(int), (void *)&height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 6, sizeof(int), (void *)&blocksize);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_hblur, 7, (blocksize+2*rad)*sizeof(float), NULL);
+  err = dt_opencl_enqueue_kernel_2d_with_local(devid, gd->kernel_sharpen_hblur, sizes, local);
   if(err != CL_SUCCESS) goto error;
-  dt_opencl_release_mem_object(dev_m);
+
+  /* vertical blur */
+  sizes[0] = ROUNDUP(width, 4);
+  sizes[1] = bheight;
+  sizes[2] = 1;
+  local[0] = 1;
+  local[1] = blocksize;
+  local[2] = 1;
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 0, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 2, sizeof(cl_mem), (void *)&dev_m);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 3, sizeof(int), (void *)&rad);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 4, sizeof(int), (void *)&width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 5, sizeof(int), (void *)&height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 6, sizeof(int), (void *)&blocksize);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_vblur, 7, (blocksize+2*rad)*sizeof(float), NULL);
+  err = dt_opencl_enqueue_kernel_2d_with_local(devid, gd->kernel_sharpen_vblur, sizes, local);
+  if(err != CL_SUCCESS) goto error;
+
+  /* mixing out and in -> out */
+  sizes[0] = ROUNDUP(width, 4);
+  sizes[1] = ROUNDUP(height, 4);
+  sizes[2] = 1;
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 0, sizeof(cl_mem), (void *)&dev_in);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 1, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 2, sizeof(cl_mem), (void *)&dev_out);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 3, sizeof(int), (void *)&width);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 4, sizeof(int), (void *)&height);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 5, sizeof(float), (void *)&d->amount);
+  dt_opencl_set_kernel_arg(devid, gd->kernel_sharpen_mix, 6, sizeof(float), (void *)&d->threshold);
+  err = dt_opencl_enqueue_kernel_2d(devid, gd->kernel_sharpen_mix, sizes);
+  if(err != CL_SUCCESS) goto error;
+
+  if (dev_m != NULL) clReleaseMemObject(dev_m);
   return TRUE;
 
 error:
@@ -129,6 +220,21 @@ error:
 }
 #endif
 
+
+void tiling_callback  (struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out, struct dt_develop_tiling_t *tiling)
+{
+  dt_iop_sharpen_data_t *d = (dt_iop_sharpen_data_t *)piece->data;
+  const int rad = MIN(MAXR, ceilf(d->radius * roi_in->scale / piece->iscale));
+
+  tiling->factor = 2;
+  tiling->overhead = 0;
+  tiling->overlap = rad;
+  tiling->xalign = 1;
+  tiling->yalign = 1;
+  return;
+}
+
+
 void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *ivoid, void *ovoid, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
   dt_iop_sharpen_data_t *data = (dt_iop_sharpen_data_t *)piece->data;
@@ -325,7 +431,7 @@ void init(dt_iop_module_t *module)
   module->params = malloc(sizeof(dt_iop_sharpen_params_t));
   module->default_params = malloc(sizeof(dt_iop_sharpen_params_t));
   module->default_enabled = 1;
-  module->priority = 644; // module order created by iop_dependencies.py, do not edit!
+  module->priority = 652; // module order created by iop_dependencies.py, do not edit!
   module->params_size = sizeof(dt_iop_sharpen_params_t);
   module->gui_data = NULL;
   dt_iop_sharpen_params_t tmp = (dt_iop_sharpen_params_t)
@@ -338,10 +444,12 @@ void init(dt_iop_module_t *module)
 
 void init_global(dt_iop_module_so_t *module)
 {
-  const int program = 2; // basic.cl, from programs.conf
+  const int program = 7; // sharpen.cl, from programs.conf
   dt_iop_sharpen_global_data_t *gd = (dt_iop_sharpen_global_data_t *)malloc(sizeof(dt_iop_sharpen_global_data_t));
   module->data = gd;
-  gd->kernel_sharpen = dt_opencl_create_kernel(program, "sharpen");
+  gd->kernel_sharpen_hblur = dt_opencl_create_kernel(program, "sharpen_hblur");
+  gd->kernel_sharpen_vblur = dt_opencl_create_kernel(program, "sharpen_vblur");
+  gd->kernel_sharpen_mix = dt_opencl_create_kernel(program, "sharpen_mix");
 }
 
 void cleanup(dt_iop_module_t *module)
@@ -355,7 +463,9 @@ void cleanup(dt_iop_module_t *module)
 void cleanup_global(dt_iop_module_so_t *module)
 {
   dt_iop_sharpen_global_data_t *gd = (dt_iop_sharpen_global_data_t *)module->data;
-  dt_opencl_free_kernel(gd->kernel_sharpen);
+  dt_opencl_free_kernel(gd->kernel_sharpen_hblur);
+  dt_opencl_free_kernel(gd->kernel_sharpen_vblur);
+  dt_opencl_free_kernel(gd->kernel_sharpen_mix);
   free(module->data);
   module->data = NULL;
 }
@@ -373,12 +483,15 @@ void gui_init(struct dt_iop_module_t *self)
   g->scale1 = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 8.0000, 0.100, p->radius, 3));
   g_object_set (GTK_OBJECT(g->scale1), "tooltip-text", _("spatial extent of the unblurring"), (char *)NULL);
   dtgtk_slider_set_label(g->scale1,_("radius"));
+  dtgtk_slider_set_accel(g->scale1,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/sharpen/radius");
   g->scale2 = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 2.0000, 0.010, p->amount, 3));
   g_object_set (GTK_OBJECT(g->scale2), "tooltip-text", _("strength of the sharpen"), (char *)NULL);
   dtgtk_slider_set_label(g->scale2,_("amount"));
+  dtgtk_slider_set_accel(g->scale2,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/sharpen/amount");
   g->scale3 = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 1.0000, 0.001, p->threshold, 3));
   g_object_set (GTK_OBJECT(g->scale3), "tooltip-text", _("threshold to activate sharpen"), (char *)NULL);
   dtgtk_slider_set_label(g->scale3,_("threshold"));
+  dtgtk_slider_set_accel(g->scale3,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/sharpen/threshold");
   gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->scale1), TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->scale2), TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(g->vbox), GTK_WIDGET(g->scale3), TRUE, TRUE, 0);
