@@ -25,6 +25,7 @@
 #include "gui/gtk.h"
 #include "dtgtk/label.h"
 #include <gdk/gdkkeysyms.h>
+#include "dtgtk/button.h"
 
 DT_MODULE(1)
 
@@ -95,6 +96,10 @@ position ()
   return 998;
 }
 
+void init_key_accels()
+{
+  gtk_button_init_accel(darktable.control->accels_capture,"<Darktable>/capture/plugins/camera/capture image(s)");
+}
 /** Property changed*/
 void property_changed_callback(GtkComboBox *cb,gpointer data)
 {
@@ -452,6 +457,7 @@ gui_init (dt_lib_module_t *self)
   gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(vbox2), TRUE, TRUE, 5);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(hbox), FALSE, FALSE, 5);
   lib->gui.button1=gtk_button_new_with_label(_("capture image(s)"));
+  gtk_button_set_accel(GTK_BUTTON(lib->gui.button1),darktable.control->accels_capture,"<Darktable>/capture/plugins/camera/capture image(s)");
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(lib->gui.button1), FALSE, FALSE, 5);
 
   g_object_set(G_OBJECT(lib->gui.tb1), "tooltip-text", _("toggle delayed capture mode"), (char *)NULL);
