@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include "dtgtk/button.h"
 
 DT_MODULE(1)
 
@@ -61,6 +62,12 @@ position ()
   return 599;
 }
 
+void init_key_accels()
+{
+  gtk_button_init_accel(darktable.control->accels_lighttable,"<Darktable>/lighttable/plugins/styles/delete");
+  gtk_button_init_accel(darktable.control->accels_lighttable,"<Darktable>/lighttable/plugins/styles/export");
+  gtk_button_init_accel(darktable.control->accels_lighttable,"<Darktable>/lighttable/plugins/styles/import");
+}
 typedef enum _styles_columns_t
 {
   DT_STYLES_COL_NAME=0,
@@ -334,22 +341,26 @@ gui_init (dt_lib_module_t *self)
 #if 0
   // TODO: Unfinished stuff
   GtkWidget *widget=gtk_button_new_with_label(_("edit"));
+  also add the shortcut
   g_signal_connect (widget, "clicked", G_CALLBACK(edit_clicked),d);
   gtk_box_pack_start(GTK_BOX (hbox),widget,TRUE,TRUE,0);
 #endif
 
   widget=gtk_button_new_with_label(_("delete"));
+  gtk_button_set_accel(GTK_BUTTON(widget),darktable.control->accels_lighttable,"<Darktable>/lighttable/plugins/styles/delete");
   g_signal_connect (widget, "clicked", G_CALLBACK(delete_clicked),d);
   g_object_set (widget, "tooltip-text", _("deletes the selected style in list above"), (char *)NULL);
   gtk_box_pack_start(GTK_BOX (hbox),widget,TRUE,TRUE,0);
   gtk_box_pack_start(GTK_BOX (self->widget),hbox,TRUE,FALSE,0);
   // Export Button
   GtkWidget *exportButton = gtk_button_new_with_label(_("export"));
+  gtk_button_set_accel(GTK_BUTTON(exportButton),darktable.control->accels_lighttable,"<Darktable>/lighttable/plugins/styles/export");
   g_object_set (exportButton, "tooltip-text", _("export the selected style into a style file"), (char *)NULL);
   g_signal_connect (exportButton, "clicked", G_CALLBACK(export_clicked),d);
   gtk_box_pack_start(GTK_BOX (hbox),exportButton,TRUE,TRUE,0);
   // Import Button
   GtkWidget *importButton = gtk_button_new_with_label(C_("styles", "import"));
+  gtk_button_set_accel(GTK_BUTTON(importButton),darktable.control->accels_lighttable,"<Darktable>/lighttable/plugins/styles/import");
   g_object_set (importButton, "tooltip-text", _("import style from a style file"), (char *)NULL);
   g_signal_connect (importButton, "clicked", G_CALLBACK(import_clicked),d);
   gtk_box_pack_start(GTK_BOX (hbox),importButton,TRUE,TRUE,0);
