@@ -83,7 +83,7 @@ init(dt_view_t *self)
       NULL);
 
   // Zoom shortcuts
-  gtk_accel_map_add_entry("<Darktable>/darkroom/zoom/close",
+  gtk_accel_map_add_entry("<Darktable>/darkroom/zoom/close-up",
                           GDK_1, GDK_MOD1_MASK);
   gtk_accel_map_add_entry("<Darktable>/darkroom/zoom/fill",
                           GDK_2, GDK_MOD1_MASK);
@@ -92,7 +92,7 @@ init(dt_view_t *self)
 
   dt_accel_group_connect_by_path(
       darktable.control->accels_darkroom,
-      "<Darktable>/darkroom/zoom/close",
+      "<Darktable>/darkroom/zoom/close-up",
       NULL);
   dt_accel_group_connect_by_path(
       darktable.control->accels_darkroom,
@@ -742,6 +742,17 @@ static void show_module_callback(GtkAccelGroup *accel_group,
   dt_iop_request_focus(module);
 }
 
+static void enable_module_callback(GtkAccelGroup *accel_group,
+                                   GObject *acceleratable,
+                                   guint keyval, GdkModifierType modifier,
+                                   gpointer data)
+
+{
+  dt_iop_module_t *module = (dt_iop_module_t*)data;
+  gboolean active= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(module->off));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(module->off), !active);
+}
+
 static void connect_closures(dt_view_t *self)
 {
   GClosure *closure;
@@ -759,7 +770,7 @@ static void connect_closures(dt_view_t *self)
   closure = g_cclosure_new(G_CALLBACK(zoom_key_accel), (gpointer)1, NULL);
   dev->closures = g_slist_prepend(dev->closures, closure);
   dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
-                                 "<Darktable>/darkroom/zoom/close",
+                                 "<Darktable>/darkroom/zoom/close-up",
                                  closure);
   closure = g_cclosure_new(G_CALLBACK(zoom_key_accel), (gpointer)2, NULL);
   dev->closures = g_slist_prepend(dev->closures, closure);
@@ -861,19 +872,30 @@ void enter(dt_view_t *self)
     GtkWidget *expander = dt_iop_gui_get_expander(module);
     module->topwidget = GTK_WIDGET(expander);
     gtk_box_pack_start(box, expander, FALSE, FALSE, 0);
-    module->show_closure = NULL;
+    module->closures = NULL;
     if(strcmp(module->op, "gamma") && !(module->flags() & IOP_FLAGS_DEPRECATED))
     {
+      GClosure* closure = NULL;
+
       // Connecting the (optional) module show accelerator
       snprintf(accelpath, 256, "<Darktable>/darkroom/plugins/%s/show plugin", module->op);
-      module->show_closure = g_cclosure_new(G_CALLBACK(show_module_callback),
-                                                       module, NULL);
+      closure = g_cclosure_new(G_CALLBACK(show_module_callback),
+                               module, NULL);
       dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
-                                     accelpath, module->show_closure);
+                                     accelpath, closure);
+      module->closures = g_list_prepend(module->closures, closure);
+
+      // Connecting the (optional) module switch accelerator
+      snprintf(accelpath, 256, "<Darktable>/darkroom/plugins/%s/enable plugin", module->op);
+      closure = g_cclosure_new(G_CALLBACK(enable_module_callback),
+                               module, NULL);
+      dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
+                                     accelpath, closure);
+      module->closures = g_list_prepend(module->closures, closure);
 
       module->showhide = dtgtk_tristatebutton_new(NULL,0);
       char filename[1024], datadir[1024];
-      dt_get_datadir(datadir, 1024);
+      dt_util_get_datadir(datadir, 1024);
       snprintf(filename, 1024, "%s/pixmaps/plugins/darkroom/%s.png", datadir, module->op);
       if(!g_file_test(filename, G_FILE_TEST_IS_REGULAR))
         snprintf(filename, 1024, "%s/pixmaps/plugins/darkroom/template.png", datadir);
@@ -988,6 +1010,13 @@ dt_dev_remove_child(GtkWidget *widget, gpointer data)
   gtk_container_remove(GTK_CONTAINER(data), widget);
 }
 
+static void
+dt_disconnect_accel_closure(gpointer data)
+{
+    dt_accel_group_disconnect(darktable.control->accels_darkroom,
+                              data);
+}
+
 void leave(dt_view_t *self)
 {
   GSList *c = ((dt_develop_t*)self->data)->closures;
@@ -1082,10 +1111,14 @@ void leave(dt_view_t *self)
     snprintf(var, 1024, "plugins/darkroom/%s/expanded", module->op);
     dt_conf_set_bool(var, gtk_expander_get_expanded (module->expander));
 
-    // disconnect the show accelerator
-    if(module->show_closure)
-      dt_accel_group_disconnect(darktable.control->accels_darkroom,
-                                module->show_closure);
+    // disconnect module closures
+    // could use this starting from gtk 2.28
+    // g_list_free_full(module->closures, dt_disconnect_accel_closure);
+    while(module->closures)
+    {
+      dt_disconnect_accel_closure(module->closures->data);
+      module->closures = g_list_delete_link(module->closures, module->closures);
+    }
 
     module->gui_cleanup(module);
     dt_iop_cleanup_module(module) ;
